"use strict";var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");Object.defineProperty(exports, "__esModule", { value: true });exports.createQuery = createQuery;exports.default = void 0;var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));



var _httpRequest = _interopRequireDefault(require("../../http-request"));
var _httpMethods = require("../../http-request/http-methods");function ownKeys(object, enumerableOnly) {var keys = Object.keys(object);if (Object.getOwnPropertySymbols) {var symbols = Object.getOwnPropertySymbols(object);if (enumerableOnly) symbols = symbols.filter(function (sym) {return Object.getOwnPropertyDescriptor(object, sym).enumerable;});keys.push.apply(keys, symbols);}return keys;}function _objectSpread(target) {for (var i = 1; i < arguments.length; i++) {var source = arguments[i] != null ? arguments[i] : {};if (i % 2) {ownKeys(Object(source), true).forEach(function (key) {(0, _defineProperty2.default)(target, key, source[key]);});} else if (Object.getOwnPropertyDescriptors) {Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));} else {ownKeys(Object(source)).forEach(function (key) {Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));});}}return target;}

/**
 * Create the base HTTP query options from RQLite API options
 * @param {Object} [options={}] The RQLite API options
 * @param {String} [options.level] The consistency level
 * @param {String} [options.pretty] Pretty print the response body
 * @param {String} [options.timings] Provide query timings
 * @param {String} [options.atomic] Treat all commands in the request as a single transaction
 * for RQLite v5 and higher
 * @param {String} [options.transaction] Treat all commands in the request as a single transaction
 * for RQLite v4 and lower
 * @returns {Object} The HTTP query
 */
function createQuery(options = {}) {const
  level = options.level,pretty = options.pretty,timings = options.timings,atomic = options.atomic,transaction = options.transaction;

  // Remove all undefined values
  const query = { level, pretty, timings, atomic, transaction };
  return Object.entries(query).reduce((acc, entry) => {const _entry = (0, _slicedToArray2.default)(
    entry, 2),key = _entry[0],val = _entry[1];
    // Only take defined values
    if (typeof val !== 'undefined') {
      acc[key] = val;
    }
    return acc;
  }, {});
}

/**
 * Base API client for RQLite which abstracts the HTTP calls
 * from the user
 */
class ApiClient extends _httpRequest.default {
  /**
   * Perform a RQLite data API get request
   * @param {String} path The path for the request i.e. /db/query
   * @param {Object} [options={}] RQLite API options
   */
  get(path, sql, options = {}) {var _superprop_getGet = () => super.get,_this = this;return (0, _asyncToGenerator2.default)(function* () {const
      useLeader = options.useLeader;
      if (!path) {
        throw new Error('The path argument is required');
      }
      return _superprop_getGet().call(_this, {
        useLeader,
        uri: path,
        httpMethod: _httpMethods.HTTP_METHOD_GET,
        query: _objectSpread(_objectSpread({}, createQuery(options)), {}, { q: sql }) });})();

  }

  /**
   * Perform a RQLite data API post request
   * @param {String} path The path for the request i.e. /db/query
   * @param {String[]|String} path The path for the request i.e. /db/query
   * @param {Object} [options={}] RQLite API options
   */
  post(path, sql, options = {}) {var _superprop_getPost = () => super.post,_this2 = this;return (0, _asyncToGenerator2.default)(function* () {const
      useLeader = options.useLeader;
      if (!path) {
        throw new Error('The path argument is required');
      }
      return _superprop_getPost().call(_this2, {
        useLeader,
        uri: path,
        httpMethod: _httpMethods.HTTP_METHOD_POST,
        query: createQuery(options),
        body: Array.isArray(sql) ? sql : [sql] });})();

  }}exports.default = ApiClient;
//# sourceMappingURL=index.js.map