"use strict";var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");Object.defineProperty(exports, "__esModule", { value: true });exports.default = exports.CONSISTENCY_LEVEL_WEAK = exports.CONSISTENCY_LEVEL_STRONG = exports.CONSISTENCY_LEVEL_NONE = exports.PATH_EXECUTE = exports.PATH_QUERY = void 0;var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));




var _client = _interopRequireDefault(require("../client"));
var _results = require("../results");function ownKeys(object, enumerableOnly) {var keys = Object.keys(object);if (Object.getOwnPropertySymbols) {var symbols = Object.getOwnPropertySymbols(object);if (enumerableOnly) symbols = symbols.filter(function (sym) {return Object.getOwnPropertyDescriptor(object, sym).enumerable;});keys.push.apply(keys, symbols);}return keys;}function _objectSpread(target) {for (var i = 1; i < arguments.length; i++) {var source = arguments[i] != null ? arguments[i] : {};if (i % 2) {ownKeys(Object(source), true).forEach(function (key) {(0, _defineProperty2.default)(target, key, source[key]);});} else if (Object.getOwnPropertyDescriptors) {Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));} else {ownKeys(Object(source)).forEach(function (key) {Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));});}}return target;}

/**
 * The RQLite query api path
 */
const PATH_QUERY = '/db/query';

/**
 * The RQLite execute api path
 */exports.PATH_QUERY = PATH_QUERY;
const PATH_EXECUTE = '/db/execute';

/**
 * Read query consistency level none which means
 * any node can respond
 */exports.PATH_EXECUTE = PATH_EXECUTE;
const CONSISTENCY_LEVEL_NONE = 'none';

/**
 * Read query consistency strong which must come from
 * the master node
 */exports.CONSISTENCY_LEVEL_NONE = CONSISTENCY_LEVEL_NONE;
const CONSISTENCY_LEVEL_STRONG = 'strong';

/**
 * Read query consistency weak which must come from
 * the master node
 */exports.CONSISTENCY_LEVEL_STRONG = CONSISTENCY_LEVEL_STRONG;
const CONSISTENCY_LEVEL_WEAK = 'weak';

/**
 * Send an RQLite query API request to the RQLite server
 * @param {String} sql The SQL string to excute on the server
 * @param {Object} [options={}] RQLite api options
 * @param {Object} [options.raw] If true return the raw http resposne from
 * RQLite response
 */exports.CONSISTENCY_LEVEL_WEAK = CONSISTENCY_LEVEL_WEAK;
function handleResponse(response, options = {}) {const
  raw = options.raw;const
  body = response.body;
  if (raw) {
    return response;
  }
  return new _results.DataResults(body);
}

/**
 * Data api client to perform RQLite queries
 */
class DataApiClient extends _client.default {
  /**
   * Send an RQLite query API request to the RQLite server
   * @param {String} sql The SQL string to excute on the server
   * @param {Object} [options={}] RQLite api options
   * @param {Object} [options.raw] If true return the raw http resposne from
   * RQLite response
   */
  query(sql, options = {}) {var _superprop_getPost = () => super.post,_superprop_getGet = () => super.get,_this = this;return (0, _asyncToGenerator2.default)(function* () {const
      level = options.level;let
      useLeader = options.useLeader;
      // Weak and strong consistency will be redirect to the master anyway
      // so skip the redirect HTTP response and got right to the master
      if (level !== CONSISTENCY_LEVEL_NONE) {
        useLeader = true;
      }
      let response;
      if (Array.isArray(sql)) {
        response = yield _superprop_getPost().call(_this, PATH_QUERY, sql, _objectSpread(_objectSpread({}, options), {}, { useLeader }));
      } else {
        response = yield _superprop_getGet().call(_this, PATH_QUERY, sql, _objectSpread(_objectSpread({}, options), {}, { useLeader }));
      }
      // If round robin is true try and balance selects across hosts when
      // the master node is not queried directly
      if (!useLeader) {
        _this.setNextActiveHostIndex();
      }
      return handleResponse(response, options);})();
  }

  /**
   * Send an RQLite execute API request to the RQLite server
   * @param {String} sql The SQL string to excute on the server
   * @param {Object} [options={}] RQLite api options
   */
  execute(sql, options = {}) {var _superprop_getPost2 = () => super.post,_this2 = this;return (0, _asyncToGenerator2.default)(function* () {
      const response = yield _superprop_getPost2().call(_this2, PATH_EXECUTE, sql, options);
      return handleResponse(response, options);})();
  }}exports.default = DataApiClient;
//# sourceMappingURL=index.js.map