"use strict";var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");Object.defineProperty(exports, "__esModule", { value: true });exports.createDefaultHeaders = createDefaultHeaders;exports.getWaitTimeExponential = getWaitTimeExponential;exports.default = exports.DEAULT_RETRY_DELAY = exports.DEAULT_TIMEOUT = void 0;var _asyncToGenerator2 = _interopRequireDefault(require("@babel/runtime/helpers/asyncToGenerator"));var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));



var _axios = _interopRequireDefault(require("axios"));
var _qs = require("qs");
var _url = require("url");
var _httpMethods = require("./http-methods");



var _contentTypes = require("./content-types");



var _errors = require("./errors");
var _retryable = require("./retryable");function ownKeys(object, enumerableOnly) {var keys = Object.keys(object);if (Object.getOwnPropertySymbols) {var symbols = Object.getOwnPropertySymbols(object);if (enumerableOnly) symbols = symbols.filter(function (sym) {return Object.getOwnPropertyDescriptor(object, sym).enumerable;});keys.push.apply(keys, symbols);}return keys;}function _objectSpread(target) {for (var i = 1; i < arguments.length; i++) {var source = arguments[i] != null ? arguments[i] : {};if (i % 2) {ownKeys(Object(source), true).forEach(function (key) {(0, _defineProperty2.default)(target, key, source[key]);});} else if (Object.getOwnPropertyDescriptors) {Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));} else {ownKeys(Object(source)).forEach(function (key) {Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));});}}return target;}





/**
 * The default timeout value
 */
const DEAULT_TIMEOUT = 30000;

/**
 * The default to retry a request using the next host in the chain
 */exports.DEAULT_TIMEOUT = DEAULT_TIMEOUT;
const DEAULT_RETRY_DELAY = 30000;

/**
 * Create default header for all HTTP requests
 * @param {Object} [headers={}] HTTP headers to send with the request
 * @returns {Object} The headers with defaults applied
 */exports.DEAULT_RETRY_DELAY = DEAULT_RETRY_DELAY;
function createDefaultHeaders(headers = {}) {const _headers$Accept =
  headers.Accept,Accept = _headers$Accept === void 0 ? _contentTypes.CONTENT_TYPE_APPLICATION_JSON : _headers$Accept;
  return _objectSpread(_objectSpread({}, headers), {}, { Accept });
}

/**
 * Clean the request path remove / from the beginning
 * @param {String} path The path to clean
 * @returns {String} The clean path
 */
function cleanPath(path) {
  return String(path).replace(/^\//, '');
}

/**
 * Returns the next wait interval, in milliseconds, using an exponential
 * backoff algorithm.
 * @param {Number} attempt The retry attempt
 * @param {Number} base The base of the exponential backoff
 * @param {Number} pow The exponential power
 * @returns {Number} The time to wait in milliseconds
 */
function getWaitTimeExponential(attempt = 0, base = 100, pow = 2) {
  if (attempt === 0) {
    return 0;
  }
  return Math.pow(pow, attempt) * base;
}

/**
 * Generic HTTP Request class which all RQLiteJS client
 * should extend for consistent communitication with an RQLite
 * server
 */
class HttpRequest {
  /**
   * The index of the host in this.hosts which will be tried
   * first before attempting other hosts
   * @type {Number}
   */


  /**
   * Whether or not the setNextActiveHostIndex() should
   * perform a round robin strategy
   */


  /**
   * The regex pattern to check if a uri is absolute or relative,
   * if it is absolute the host is not appended
   */


  /**
   * A list of hosts that are tried in round robin fashion
   * when certain HTTP responses are received
   * @type {String[]}
   */


  /**
   * @type {import('http').Agent} The http agent if it is set
   */


  /**
   * @type {import('https').Agent} The https agent if it is set
   */


  /**
   * The host list index of the leader node defaults
   * to the first host
   * @type {Number}
   */


  /**
   * Http error codes which are considered retryable
   * @type {Set}
   */


  /**
   * Http status codes which are considered retryable
   * @type {Set}
   */


  /**
   * Http methods which are considered retryable
   * @type {Set}
   */


  /**
   * The exponential backoff base for retries
   */


  /**
   * Construtor for HttpRequest
   * @param {String[]|String} hosts An array of RQLite hosts or a string
   * that will be split on "," to create an array of hosts, the first
   * host will be tried first when there are multiple hosts
   * @param {Object} [options={}] Additional options
   * @param {Boolean} [options.activeHostRoundRobin=true] If true this.setNextActiveHostIndex()
   * will perform a round robin when called
   * @param {import('http').Agent} [options.httpAgent] An option http agent, useful for
   * keepalive pools using plain HTTP
   * @param {import('https').Agent} [options.httpsAgent] An option http agent, useful
   * for keepalive pools using SSL
   * @param {Set} [options.retryableErrorCodes] The list of retryable error codes
   * @param {Set} [options.retryableStatusCodes] The list of retryable http status codes
   * @param {Set} [options.retryableHttpMethods] The list of retryable http methods
   * @param {Number} [options.exponentailBackoffBase] The value for exponentail backoff base
   * for retry exponential backoff
   */
  constructor(hosts, options = {}) {(0, _defineProperty2.default)(this, "activeHostIndex", 0);(0, _defineProperty2.default)(this, "activeHostRoundRobin", true);(0, _defineProperty2.default)(this, "absoluteUriPattern", /^https?:\/\//);(0, _defineProperty2.default)(this, "hosts", []);(0, _defineProperty2.default)(this, "httpAgent", void 0);(0, _defineProperty2.default)(this, "httpsAgent", void 0);(0, _defineProperty2.default)(this, "leaderHostIndex", 0);(0, _defineProperty2.default)(this, "retryableErrorCodes", _retryable.RETRYABLE_ERROR_CODES);(0, _defineProperty2.default)(this, "retryableStatusCodes", _retryable.RETRYABLE_STATUS_CODES);(0, _defineProperty2.default)(this, "retryableHttpMethods", _retryable.RETRYABLE_HTTP_METHODS);(0, _defineProperty2.default)(this, "exponentailBackoffBase", 100);
    this.setHosts(hosts);
    if (this.getTotalHosts() === 0) {
      throw new Error('At least one host must be provided');
    }const _options$activeHostRo =








    options.activeHostRoundRobin,activeHostRoundRobin = _options$activeHostRo === void 0 ? true : _options$activeHostRo,httpAgent = options.httpAgent,httpsAgent = options.httpsAgent,retryableErrorCodes = options.retryableErrorCodes,retryableStatusCodes = options.retryableStatusCodes,retryableHttpMethods = options.retryableHttpMethods,exponentailBackoffBase = options.exponentailBackoffBase;
    if (typeof activeHostRoundRobin !== 'undefined') {
      this.setActiveHostRoundRobin(activeHostRoundRobin);
    }
    if (typeof httpAgent !== 'undefined') {
      this.setHttpAgent(httpAgent);
    }
    if (typeof httpsAgent !== 'undefined') {
      this.setHttpsAgent(httpsAgent);
    }
    if (retryableErrorCodes instanceof Set) {
      this.setRetryableErrorCodes(retryableErrorCodes);
    }
    if (retryableStatusCodes instanceof Set) {
      this.setRetryableStatusCodes(retryableStatusCodes);
    }
    if (retryableHttpMethods instanceof Set) {
      this.setRetryableHttpMethods(retryableHttpMethods);
    }
    if (Number.isFinite(exponentailBackoffBase)) {
      this.setExponentailBackoffBase(exponentailBackoffBase);
    }
  }

  /**
   * Set the exponentail backoff base
   * @param {Number} exponentailBackoffBase
   */
  setExponentailBackoffBase(exponentailBackoffBase) {
    this.exponentailBackoffBase = exponentailBackoffBase;
  }

  /**
   * Get the exponentail backoff base
   * @return {Number} The exponentail backoff base
   */
  getExponentailBackoffBase() {
    return this.exponentailBackoffBase;
  }

  /**
   * Set the retryable error codes
   * @param {Set} retryableErrorCodes
   */
  setRetryableErrorCodes(retryableErrorCodes) {
    this.retryableErrorCodes = retryableErrorCodes;
  }

  /**
   * Get the retryable error codes
   * @returns {Set}
   */
  getRetryableErrorCodes() {
    return this.retryableErrorCodes;
  }

  /**
   * Set the retryable status codes
   * @param {Set} retryableStatusCodes
   */
  setRetryableStatusCodes(retryableStatusCodes) {
    this.retryableStatusCodes = retryableStatusCodes;
  }

  /**
   * Get the retryable status codes
   * @returns {Set}
   */
  getRetryableStatusCodes() {
    return this.retryableStatusCodes;
  }

  /**
   * Set the retryable http methods
   * @param {Set} retryableHttpMethods
   */
  setRetryableHttpMethods(retryableHttpMethods) {
    this.retryableHttpMethods = retryableHttpMethods;
  }

  /**
   * Get the retryable http methods
   * @returns {Set}
   */
  getRetryableHttpMethods() {
    return this.retryableHttpMethods;
  }

  /**
   * Set an http agent which is useful for http keepalive requests
   * @param {import('http').Agent} httpAgent An http agent
   */
  setHttpAgent(httpAgent) {
    this.httpAgent = httpAgent;
  }

  /**
   * Get the set http agent
   * @returns {import('http').Agent|undefined} The https agent if it is set
   */
  getHttpAgent() {
    return this.httpAgent;
  }

  /**
   * Set an https agent which is useful for https keepalive requests
   * @param {import('https').Agent} httpsAgent An https agent
   */
  setHttpsAgent(httpsAgent) {
    this.httpsAgent = httpsAgent;
  }

  /**
   * Get the set https agent
   * @returns {import('https').Agent|undefined} The https agent if it is set
   */
  getHttpsAgent() {
    return this.httpsAgent;
  }

  /**
   * Set the list of hosts
   * @param {String[]|String} hosts An array of RQLite hosts or a string
   * that will be split on "," to create an array of hosts, the first
   * host will be tried first when there are multiple hosts
   */
  setHosts(hosts) {
    this.hosts = !Array.isArray(hosts) ? String(hosts).split(',') : hosts;
    this.hosts = this.hosts.reduce((acc, v) => {
      // Remove trailing slashed from hosts
      const host = String(v).trim().replace(/\/$/, '');
      if (!host) {
        return acc;
      }
      return acc.concat(host);
    }, []);
  }

  /**
   * Get the list of hosts
   * @returns {String[]} The list of hosts
   */
  getHosts() {
    return this.hosts;
  }

  /**
   * Given a host string find the index of that host in the hosts
   * @param {String} host A host to find in hosts
   * @returns {Number} The found host index or -1 if not found
   */
  findHostIndex(host) {
    const parsedHostToFind = (0, _url.parse)(host);
    return this.getHosts().findIndex(v => {
      const parsedHost = (0, _url.parse)(v);
      // Find a host where all the parsed fields match the requested host
      return ['hostname', 'protocol', 'port', 'path'].every(field => parsedHostToFind[field] === parsedHost[field]);
    });
  }

  /**
   * Get the current active host from the hosts array
   * @param {Boolean} useLeader If true use the first host which is always
   * the master, this is prefered for write operations
   * @returns {String} The active host
   */
  getActiveHost(useLeader) {
    // When useLeader is true we should just use the first host
    const activeHostIndex = useLeader ? this.getLeaderHostIndex() : this.getActiveHostIndex();
    return this.getHosts()[activeHostIndex];
  }

  /**
   * Set the active host index with check based on this.hosts
   * @param {Number} activeHostIndex The index
   * @returns {Number} The active host index
   */
  setActiveHostIndex(activeHostIndex) {
    if (!Number.isFinite(activeHostIndex)) {
      throw new Error('The activeHostIndex should be a finite number');
    }
    const totalHosts = this.getTotalHosts();
    if (activeHostIndex < 0) {
      // Don't allow an index less then zero
      this.activeHostIndex = 0;
    } else if (activeHostIndex >= totalHosts) {
      // Don't allow an index greater then the length of the hosts
      this.activeHostIndex = totalHosts - 1;
    } else {
      this.activeHostIndex = activeHostIndex;
    }
    return this.activeHostIndex;
  }

  /**
   * Get the host index for the leader node
   * @returns {Number} The host index for the leader node
   */
  getLeaderHostIndex() {
    return this.leaderHostIndex;
  }

  /**
   * Set the index in the host array for the leader node
   * @param {Number} leaderHostIndex The index of the host that is the leader node
   * @returns {Number} The host index for the leader node
   */
  setLeaderHostIndex(leaderHostIndex) {
    if (!Number.isFinite(leaderHostIndex)) {
      throw new Error('The leaderHostIndex should be a finite number');
    }
    const totalHosts = this.getTotalHosts();
    if (leaderHostIndex < 0) {
      this.leaderHostIndex = 0;
    } else if (leaderHostIndex > totalHosts) {
      this.leaderHostIndex = totalHosts - 1;
    } else {
      this.leaderHostIndex = leaderHostIndex;
    }
    return this.leaderHostIndex;
  }

  /**
   * Get the active host index
   * @returns {Number} The active host index
   */
  getActiveHostIndex() {
    return this.activeHostIndex;
  }

  /**
   * Set active host round robin value
   * @param {Boolean} activeHostRoundRobin If true setActiveHostIndex() will
   * perform a round robin
   */
  setActiveHostRoundRobin(activeHostRoundRobin) {
    if (typeof activeHostRoundRobin !== 'boolean') {
      throw new Error('The activeHostRoundRobin argument must be boolean');
    }
    this.activeHostRoundRobin = activeHostRoundRobin;
  }

  /**
   * Get active host round robin value
   * @returns {Boolean} The value of activeHostRoundRobin
   */
  getActiveHostRoundRobin() {
    return this.activeHostRoundRobin;
  }

  /**
   * Get the next active host index
   * @param {Number} [activeHostIndex] An optional paramater to provide the active host index
   * @returns {Number} The next active host index which will wrap around to zero
   */
  getNextActiveHostIndex(activeHostIndex = this.getActiveHostIndex()) {
    const totalHosts = this.getTotalHosts();
    const nextIndex = activeHostIndex + 1;
    // If we are past the last index start back over at 1
    if (totalHosts === nextIndex) {
      return 0;
    }
    return nextIndex;
  }

  /**
   * Set the active host index to the next host using a
   * round robin strategy
   */
  setNextActiveHostIndex() {
    // Don't bother if we only have one host
    if (!this.getActiveHostRoundRobin()) {
      return;
    }
    const totalHosts = this.getTotalHosts();
    if (this.getActiveHostRoundRobin() && totalHosts <= 1) {
      return;
    }
    this.setActiveHostIndex(this.getNextActiveHostIndex());
  }

  /**
   * Get the total number of hosts
   * @returns {Number} The total number of hosts
   */
  getTotalHosts() {
    return this.getHosts().length;
  }

  /**
   * Returns whether or not the uri passes a test for this.absoluteUriPattern
   * @returns {Boolean} True if the path is absolute
   */
  uriIsAbsolute(uri) {
    return this.absoluteUriPattern.test(uri);
  }

  /**
   * Returns true when the HTTP request is retryable
   * @param {Object} options The options
   * @param {Number} options.statusCode The HTTP status code
   * @param {String} options.errorCode The error code
   * @param {String} options.httpMethod The http method
   * @returns {Boolean} True if the request is retry able
   */
  requestIsRetryable(options = {}) {const

    statusCode =


    options.statusCode,errorCode = options.errorCode,httpMethod = options.httpMethod;
    // Honor strictly the http method
    if (!this.getRetryableHttpMethods().has(httpMethod)) {
      return false;
    }
    if (statusCode && this.getRetryableStatusCodes().has(statusCode)) {
      return true;
    }
    if (errorCode && this.getRetryableErrorCodes().has(errorCode)) {
      return true;
    }
    return false;
  }

  /**
   * Perform an HTTP request using the provided options
   * @param {Object} [options={}] Options for the HTTP client
   * @param {Object} [options.auth] A object for user authentication
   * @param {String} [options.auth.username] The username for authentication
   * @param {String} [options.auth.password] The username for authentication
   * @param {Object} [options.body] The body of the HTTP request
   * @param {Object} [options.headers={}] HTTP headers to send with the request
   * @param {String} [options.httpMethod=HTTP_METHOD_GET] The HTTP method for the request
   * i.e. GET or POST
   * @param {Object} [options.query] An object with the query to send with the HTTP request
   * @param {Boolean} [options.stream=false] When true the returned value is a request object with
   * stream support instead of a request-promise result
   * @param {Number} [options.timeout=DEAULT_TIMEOUT] Optional timeout to override default
   * @param {String} options.uri The uri for the request which can be a relative path to use
   * the currently active host or a full i.e. http://localhost:4001/db/query which is used
   * literally
   * @param {Boolean} [options.useLeader=false] When true the request will use the master host, the
   * first host in this.hosts, this is ideal for write operations to skip the redirect
   * @param {Number} [options.retries] The number of retries, defaults to the number of
   * hosts times 3
   * @param {Number} [options.maxRedirects=10] The maximum number of HTTP redirects to follow before
   * throwing an error
   * @param {Number} [options.attempt=0] The current attempt count when retrying or redirecting
   * @param {Number} [options.retryAttempt=0] The current attempt based on retry logic
   * @param {Number} [options.redirectAttempt=0] The current attempt based on redirect logic
   * @param {Number} [options.attemptHostIndex] When in a retry state the host index of the last
   * attempt which is used to get the next host index
   * @param {import('http').Agent} [options.httpAgent] An option http agent, useful for
   * keepalive pools using plain HTTP
   * @param {import('https').Agent} [options.httpsAgent] An option http agent, useful
   * for keepalive pools using SSL
   * @returns {Promise<{status: Number, body: Object|String}>} An object with a status and body
   * property when stream is false and a stream when the stream option is true
   * @throws {ERROR_HTTP_REQUEST_MAX_REDIRECTS} When the maximum number of redirect has been reached
   */
  fetch(options = {}) {var _this = this;return (0, _asyncToGenerator2.default)(function* () {const

      auth =
















      options.auth,body = options.body,_options$headers = options.headers,headers = _options$headers === void 0 ? {} : _options$headers,_options$httpMethod = options.httpMethod,httpMethod = _options$httpMethod === void 0 ? _httpMethods.HTTP_METHOD_GET : _options$httpMethod,query = options.query,_options$stream = options.stream,stream = _options$stream === void 0 ? false : _options$stream,_options$timeout = options.timeout,timeout = _options$timeout === void 0 ? DEAULT_TIMEOUT : _options$timeout,_options$useLeader = options.useLeader,useLeader = _options$useLeader === void 0 ? false : _options$useLeader,_options$retries = options.retries,retries = _options$retries === void 0 ? _this.getTotalHosts() * 3 : _options$retries,_options$maxRedirects = options.maxRedirects,maxRedirects = _options$maxRedirects === void 0 ? 10 : _options$maxRedirects,_options$attempt = options.attempt,attempt = _options$attempt === void 0 ? 0 : _options$attempt,_options$retryAttempt = options.retryAttempt,retryAttempt = _options$retryAttempt === void 0 ? 0 : _options$retryAttempt,_options$redirectAtte = options.redirectAttempt,redirectAttempt = _options$redirectAtte === void 0 ? 0 : _options$redirectAtte,attemptHostIndex = options.attemptHostIndex,_options$exponentailB = options.exponentailBackoffBase,exponentailBackoffBase = _options$exponentailB === void 0 ? _this.getExponentailBackoffBase() : _options$exponentailB,_options$httpAgent = options.httpAgent,httpAgent = _options$httpAgent === void 0 ? _this.getHttpAgent() : _options$httpAgent,_options$httpsAgent = options.httpsAgent,httpsAgent = _options$httpsAgent === void 0 ? _this.getHttpsAgent() : _options$httpsAgent;
      // Honor the supplied attemptHostIndex or get the active host
      const activeHost = Number.isFinite(attemptHostIndex) ? _this.getHosts()[attemptHostIndex] :
      _this.getActiveHost(useLeader);let
      uri = options.uri;
      if (!uri) {
        throw new Error('The uri option is required');
      }
      uri = _this.uriIsAbsolute(uri) ? uri : `${activeHost}/${cleanPath(uri)}`;
      try {
        const response = yield (0, _axios.default)({
          url: uri,
          auth: auth && typeof auth === 'object' ? {
            username: auth.user || auth.username,
            password: auth.pass || auth.password } :
          undefined,
          data: body,
          maxRedirects: 0, // Handle redirects manually to allow reposting data
          headers: createDefaultHeaders(headers),
          responseType: stream ? 'stream' : 'json',
          method: httpMethod,
          params: query,
          timeout,
          httpsAgent,
          httpAgent,
          paramsSerializer(params) {
            return (0, _qs.stringify)(params, { arrayFormat: 'brackets' });
          } });

        if (stream) {
          return response.data;
        }
        return {
          body: response.data,
          status: response.status };

      } catch (e) {const _e$response =
        e.response,response = _e$response === void 0 ? {} : _e$response,errorCode = e.code;const
        responseStatus = response.status,_response$headers = response.headers,responseHeaders = _response$headers === void 0 ? {} : _response$headers;
        // Check if the error was a redirect
        const retryable = _this.requestIsRetryable({
          statusCode: responseStatus,
          errorCode,
          httpMethod });

        // Save the next active host index and pass it to retry manually
        let nextAttemptHostIndex = Number.isFinite(attemptHostIndex) ? attemptHostIndex :
        _this.getActiveHostIndex();
        nextAttemptHostIndex += 1;
        // We go past the last index start from zero
        if (nextAttemptHostIndex === _this.getTotalHosts()) {
          nextAttemptHostIndex = 0;
        }
        // First check if this is a redirect error
        if (responseStatus === 301 || responseStatus === 302) {
          // We maxed out on redirect attempts
          if (redirectAttempt >= maxRedirects) {
            throw (0, _errors.ERROR_HTTP_REQUEST_MAX_REDIRECTS)(`The maximum number of redirects ${maxRedirects} has been reached`);
          }
          const location = typeof responseHeaders === 'object' ? responseHeaders.location : undefined;
          // If we were asked to use the leader, but got redirect the leader moved so remember it
          if (useLeader) {
            const newLeaderHostIndex = _this.findHostIndex(location);
            // If the redirect exists in the hosts list remember it for next time
            if (newLeaderHostIndex > -1) {
              _this.setLeaderHostIndex(newLeaderHostIndex);
            }
          }
          return _this.fetch(_objectSpread(_objectSpread({},
          options), {}, {
            uri: location,
            attempt: attempt + 1,
            redirectAttempt: redirectAttempt + 1,
            attemptHostIndex: nextAttemptHostIndex }));

        }
        if (retryable && retryAttempt < retries) {
          const waitTime = getWaitTimeExponential(retryAttempt, exponentailBackoffBase);
          const delayPromise = new Promise(resolve => setTimeout(resolve, waitTime));
          yield delayPromise;
          return _this.fetch(_objectSpread(_objectSpread({},
          options), {}, {
            attempt: attempt + 1,
            retryAttempt: retryAttempt + 1,
            attemptHostIndex: nextAttemptHostIndex }));

        }
        throw e;
      }})();
  }

  /**
   * Perform an HTTP GET request
   * @param {Object} [options={}] The options
   * @see this.fetch() for options
   */
  get(options = {}) {var _this2 = this;return (0, _asyncToGenerator2.default)(function* () {
      return _this2.fetch(_objectSpread(_objectSpread({}, options), {}, { httpMethod: _httpMethods.HTTP_METHOD_GET }));})();
  }

  /**
   * Perform an HTTP POST request
   * @param {Object} [options={}] The options
   * @see this.fetch() for options
   */
  post(options = {}) {var _this3 = this;return (0, _asyncToGenerator2.default)(function* () {
      return _this3.fetch(_objectSpread(_objectSpread({}, options), {}, { httpMethod: _httpMethods.HTTP_METHOD_POST }));})();
  }}exports.default = HttpRequest;
//# sourceMappingURL=index.js.map